import React, { PropTypes } from 'react';
import Editor from 'draft-js-plugins-editor';
import createEmojiPlugin from 'draft-js-emoji-plugin';
import createEntityPropsPlugin from 'draft-js-entity-props-plugin';
import createHashtagPlugin from 'draft-js-hashtag-plugin';
import createLinkifyPlugin from 'draft-js-linkify-plugin';
import createTablePlugin from 'draft-js-table-plugin';
import createToolbarPlugin from 'draft-js-toolbar-plugin';
import { AtomicBlockUtils, EditorState, RichUtils, Entity, convertToRaw, convertFromRaw } from 'draft-js';
import Sidebar from './Sidebar';
import Media from './Media';

const emojiPlugin = createEmojiPlugin();
const { EmojiSuggestions } = emojiPlugin;
const entityPlugin = createEntityPropsPlugin();
const hashtagPlugin = createHashtagPlugin();
const linkifyPlugin = createLinkifyPlugin({
  target: '_blank'  // default is '_self'
});
const toolbarPlugin = createToolbarPlugin({
  __toolbarHandler: {
    add: props => console.log('Add toolbar', props),
    remove: props => console.log('Remove toolbar', props),
  }, textActions: [
    {
      button: <span>code</span>,
      key: 'in-code',
      label: 'Inline code',
      active: (block, editorState) => block.get('type') === 'CODE',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleInlineStyle(
        editorState,
        'CODE'
      )),
    },
    {
      button: <span>H1</span>,
      key: 'H1',
      label: 'Header 1',
      active: (block, editorState) => block.get('type') === 'header-one',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'header-one'
      )),
    },
    {
      button: <span>H2</span>,
      key: 'H2',
      label: 'Header 2',
      active: (block, editorState) => block.get('type') === 'header-two',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'header-two'
      )),
    },
    {
      button: <span>H3</span>,
      key: 'H3',
      label: 'Header 3',
      active: (block, editorState) => block.get('type') === 'header-three',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'header-three'
      )),
    },
    {
      button: <span>H4</span>,
      key: 'H4',
      label: 'Header 4',
      active: (block, editorState) => block.get('type') === 'header-four',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'header-four'
      )),
    }, {
      button: <span>H5</span>,
      key: 'H5',
      label: 'Header 5',
      active: (block, editorState) => block.get('type') === 'header-five',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'header-five'
      )),
    }, {
      button: <span>quote</span>,
      key: 'quote',
      label: 'Blockquote',
      active: (block, editorState) => block.get('type') === 'blockquote',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'blockquote'
      )),
    }, {
      button: <span>list</span>,
      key: 'u-list',
      label: 'Unordered list',
      active: (block, editorState) => block.get('type') === 'unordered-list-item',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'unordered-list-item'
      )),
    }, {
      button: <span>list</span>,
      key: 'o-list',
      label: 'Ordered list',
      active: (block, editorState) => block.get('type') === 'ordered-list-item',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'ordered-list-item'
      )),
    }, {
      button: <span>code</span>,
      key: 'code',
      label: 'Code block',
      active: (block, editorState) => block.get('type') === 'code-block',
      toggle: (block, action, editorState, setEditorState) => setEditorState(RichUtils.toggleBlockType(
        editorState,
        'code-block'
      )),
    }
  ]
});
const tablePlugin = createTablePlugin({
  Editor
});

const { tableCreator } = tablePlugin;

const plugins = [
  emojiPlugin,
  entityPlugin,
  hashtagPlugin,
  linkifyPlugin,
  tablePlugin,
  toolbarPlugin,
];

class RichTextEditor extends React.Component {
  constructor(props) {
    super(props);

    if (this.props.content) {
      let contentState = convertFromRaw(this.props.content);

      this.state = {
        editorState: EditorState.createWithContent(contentState),
      };
    } else {
      this.state = {
        editorState: EditorState.createEmpty(),
      };
    }

    this.focus = () => this.refs.editor.focus();
    this.onChange = (editorState) => this.setState({ editorState });
    this.handleKeyCommand = this.handleKeyCommand.bind(this);

    this.convertToRaw = () => {
      const { editorState } = this.state;
      let contentState = editorState.getCurrentContent();
      return convertToRaw(contentState);
    };
  }

  handleKeyCommand(command) {
    const { editorState } = this.state;
    const newState = RichUtils.handleKeyCommand(editorState, command);
    if (newState) {
      this.onChange(newState);
      return true;
    }
    return false;
  }

  blockRenderer(block) {
    if (block.getType() === 'atomic') {
      return { component: Media, editable: false };
    }
    return null;
  }

  render() {
    const { editorState } = this.state;
    const { filePickerKey } = this.props;

    return (
      <div id="rich-text-editor" className="callout">
        <Sidebar
          onChange={this.onChange}
          filePickerKey={filePickerKey}
          editorState={editorState}
        />

        <div
          style={{ marginLeft: '50px', minHeight: '50vh' }}
          onClick={this.focus}
        >
          <Editor
            blockRendererFn={this.blockRenderer}
            editorState={editorState}
            handleKeyCommand={this.handleKeyCommand}
            onChange={this.onChange}
            placeholder="Tell a story..."
            ref="editor"
            plugins={plugins}
            spellCheck={true}
          />
          <tableCreator />
          <EmojiSuggestions />

        </div>
      </div>
    );
  }
}

RichTextEditor.props = {
  filePickerKey: PropTypes.string.isRequired,
};

export default RichTextEditor;
